// PTHistogram.java

import swarm.Globals;
import swarm.collections.ListImpl;
import javax.swing.JFrame;
import ptolemy.plot.Histogram;

/**
 * PTHistogram contains three contructors to develop histogram graphs
 *
 * @author Dario Landini (light changes by Pietro Terna; adapted to 
 * PTHistogramPlottable interface by Michele Sonnessa)
 */
public class PTHistogram extends Histogram{

    public boolean useSecondList=false;
    public boolean useThirdList=false;

    private JFrame myFrame = null;

    /** The Lists passed through the constructors*/
    ListImpl fL;
    ListImpl sL;
    ListImpl tL;
    /** the temporary address of an existing unit, viewed following the
        PTHistogramPlottable interface */
    PTHistogramPlottable unit;

    /**
     * First class constructor
     * @param xpos the horizontal position in pixels (left side is 0)
     * @param ypos the vertical position in pixels (upper side is 0)
     * @param title the title of the histogram
     * @param xLabel the label on the X axis
     * @param yLabel the label on the Y axis
     * @param xRange the right dimension of the histogram
     * @param xRange the top dimension of the histogram
     * @param firstList a data list
     * @param caption0 the caption in legend
     */
    public PTHistogram(int xpos, int ypos,
		       String title, String xLabel, String yLabel, 
		       double xRange, double yRange, ListImpl firstList,
		       String caption0) {

        //create the Frame for the histogram graph
        myFrame = new JFrame();
        //estabilish the title of graph with ptolemy
        setTitle(title);
        //pass the same title to the window's frame
        myFrame.setTitle(title);
        //estabilish the axes' labels with ptolemy
        setXLabel(xLabel);
        setYLabel(yLabel);
        //the lenght of x & y axes
        setXRange(0,xRange);
        setYRange(0,yRange);
        //the right position of bars and their widht
        setBinOffset(1);
        setBinWidth(0.5);
        //the legend
        addLegend(0, caption0);

        fL = firstList;
        //create the "fill" button to resize graph
        setButtons(true);
        //fix the window's dimensions according to x & y range
        int windowWidth = 400;
        int windowHigh = 300;
	if(xRange > 18){
            windowWidth = 700;
	}
	if(xRange<5){
            windowWidth = 300;
	}
        myFrame.setBounds(xpos,ypos,windowWidth,windowHigh);
	
        //add the histogram to the frame
        myFrame.getContentPane().add(this);
        //show the frame
        myFrame.show();
    }

    /**
     * Second class constructor
     * @param xpos the horizontal position in pixels (left side is 0)
     * @param ypos the vertical position in pixels (upper side is 0)
     * @param title the title of the histogram
     * @param xLabel the label on the X axis
     * @param yLabel the label on the Y axis
     * @param xRange the right dimension of the histogram
     * @param xRange the top dimension of the histogram
     * @param firstList a first data list
     * @param caption0 the first caption in legend
     * @param secondList a second data list
     * @param caption1 the second caption in legend
     */
    public PTHistogram(int xpos, int ypos,
		       String title, String xLabel, String yLabel, 
		       double xRange, double yRange, 
		       ListImpl firstList, String caption0,
		       ListImpl secondList, String caption1) {
	
        useSecondList = true; // if we are using this constructor the
	                      // second list is activated

        myFrame = new JFrame();
        setTitle(title);
        myFrame.setTitle(title);
	
        setXLabel(xLabel);
        setYLabel(yLabel);
        setXRange(0,xRange);
        setYRange(0,yRange);
	
        setBinOffset(1);
        setBinWidth(0.5);

        addLegend(0, caption0);
        addLegend(1, caption1);
	
        fL = firstList;
        sL = secondList;
	
        setButtons(true);
	
        int windowWidth = 400;
        int windowHigh = 300;
	if(xRange > 18){
            windowWidth = 700;
	}
	if(xRange<5){
            windowWidth = 300;
	}
        myFrame.setBounds(xpos,ypos,windowWidth,windowHigh);
	
        myFrame.getContentPane().add(this);
        myFrame.show();
  }
    
    /**
     * Third class constructor
     * @param xpos the horizontal position in pixels (left side is 0)
     * @param ypos the vertical position in pixels (upper side is 0)
     * @param title the title of the histogram
     * @param xLabel the label on the X axis
     * @param yLabel the label on the Y axis
     * @param xRange the right dimension of the histogram
     * @param xRange the top dimension of the histogram
     * @param firstList a first data list
     * @param caption0 the first caption in legend
     * @param secondList a second data list
     * @param caption1 the secondf caption in legend
     * @param thirdList a third data list
     * @param caption1 the third caption in legend
     */
    
    public PTHistogram(int xpos, int ypos,
		       String title, String xLabel, String yLabel, 
		       double xRange, double yRange, 
		       ListImpl firstList, String caption0, 
		       ListImpl secondList, String caption1,
		       ListImpl thirdList, String caption2) {
	
        useSecondList = true; // if we are using this constructor the
	                      // second list is activated
        useThirdList =  true; // if we are using this constructor the
	                      // third list is activated

        myFrame = new JFrame();
        setTitle(title);
        myFrame.setTitle(title);
	
        setXLabel(xLabel);
        setYLabel(yLabel);
        setXRange(0,xRange);
        setYRange(0,yRange);
	
        setBinOffset(1);
        setBinWidth(0.5);
	
        addLegend(0, caption0);
        addLegend(1, caption1);
        addLegend(2, caption2);
	
        fL = firstList;
        sL = secondList;
        tL = thirdList;
	
        setButtons(true);
	
        int windowWidth = 400;
        int windowHigh = 300;
	if(xRange > 18){
	    windowWidth = 700;
	}
	if(xRange<5){
            windowWidth = 300;
	}
        myFrame.setBounds(xpos,ypos,windowWidth,windowHigh);
	
        myFrame.getContentPane().add(this);
        myFrame.show();
    }
    
  /**
   * this method should be called by the Observer to paint histogram bars,
   * at every schedule. It paints a point for each number contained
   * in the lists of the single object.
   */
    public void addPoints(){
	//it clears graph to repaint it, but it doesn't forget the
	//window instructions
	clear(false);
	// scroll the first list
	for (int i=0;i<fL.getCount();i++){
	    unit = (PTHistogramPlottable) fL.atOffset(i);
	    String singleLabels = unit.getLabel();
	    //add the labels on x axis
	    addXTick(singleLabels , i);

	    double valueToPlot = unit.getValueToPlot();
	    //addPoint according to the valueToPlot
	    for(int j=0;j<valueToPlot;j++)
		addPoint(0, (double) i);
	}       
	

	if(useSecondList){
	    // scroll the second list
	    for (int i=0;i<fL.getCount();i++){
		unit = (PTHistogramPlottable) sL.atOffset(i);
		double valueToPlot = unit.getValueToPlot();
		//addPoint according to the valueToPlot
		for(int j=0;j<valueToPlot;j++)
		    addPoint(1, (double) i);     
	    }
	}

	if(useThirdList){
	    // scroll the third list
	    for (int i=0;i<fL.getCount();i++){
		unit = (PTHistogramPlottable) tL.atOffset(i);
		double valueToPlot = unit.getValueToPlot();
		//addPoint according to the valueToPlot
		for(int j=0;j<valueToPlot;j++)
		    addPoint(2, (double) i);     
	    }
	}

	//repaint the window at each time and execute clear instruction
	repaint();
    }
}



