//  StandAloneBatchAssembler.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;

/**
 * The  StandAloneBatchAssembler class instances are unit 
 * assembling stand alone batch processes
 *
 * @author Pietro Terna 
 */
public class StandAloneBatchAssembler extends SwarmObjectImpl implements 
    PTHistogramPlottable{

    /** the unit we are assembling for */
    Unit myUnit;
    /** the list of the order to be assembled with a stand alone batch 
	process */
    public ListImpl waitingList;

    /**
     * the constructor for StandAloneBatchAssembler
     */
    public StandAloneBatchAssembler (Zone aZone)
    {
	// Call the constructor for the parent class.
	super(aZone);

  	// the list of the stand alone batch processes to be assembled
	waitingList = new ListImpl (getZone());
    }

    /** setting the unit we are assembling for */
    public void setUnit(Unit u){
	myUnit = u;
    }

    /** adding an order to the waitingList */
    public boolean setStandAloneBatchWaitingList(Order anOrder){
	waitingList.addLast(anOrder);
	return true;
    }

    /** verifying stand alone batches and eliminating the orders from our
     *  waitingList in scheduling */
    public void checkingStandAloneBatchAndFreeingOrders(){
	// empty method, included for simmetry reasons and for future use
    }

    /**
     * return the waiting list length
     */
    public int getWaitingListLength()
    {
	return waitingList.getCount();
    }

    /**
     * checking if an order is in the waitingList
     */
    public boolean thisOrderIsInTheWaitingList(Order o){

	// the result is true if o is a member of waitingList; false in
	// the opposite case
	return waitingList.contains(o);
    }

    /**
     * removing the order from the waitingList
     */
    public void removeThisOrderFromTheWaitingList(Order o){

	waitingList.remove(o);
    }


    /**
     * PTHistogramPlottable interface method: getLabel() 
     */
    public String getLabel(){
	return "";
    }

    /**
     * PTHistogramPlottable interface method: getValueToPlot() 
     */
    public double getValueToPlot(){
	return (double) getWaitingListLength();
    }

}



